#!/usr/bin/perl

#                           
# indexer.pl - The Perlfect Indexer
# version 1.0
# 
# (c) 1998 Giorgos Zervas, Perlfect Solutions                        
# giorgos@perlfect.com
# http://perlfect.co
#
# 15 November 1998
#
# requires the following perl modules:
# Data::Dumper
#
# suggested scripts:
# search.pl >=v1.0
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

use Data::Dumper;

#######################################################################
# User configurable variables start here

# Where the indexer should start...
my $base_dir="/home/nick/perlfect/www";

# The index file (unless $debug below is set to 1 this file is not stored)
my $index_file='/home/nick/perlfect/www/cgi-bin/data/contents.index';
# The flattened index file
my $flat_index_file='/home/nick/perlfect/www/cgi-bin/data/word.index';
# The file which describes the assignments of documents to numbers
my $file_index_file='/home/nick/perlfect/www/cgi-bin/data/file.index';

# This is a comma separated list of the directories you want the indexer
# to exclude, the same applies to second list for individual files 
my @exclude_dirs=(
		  "/home/nick/perlfect/www/ssi",
		  "/home/nick/perlfect/www/cgi-bin",
		 );

my @exclude_files=(
		   "/home/nick/perlfect/www/services/quote_res.shtml",
		  );


# The following values are well weighted for most sites
# Changing them may occur in unpredicatable results

# How important it is to find a keyword in the document description meta
my $meta_weight=5;
# How important it is to find a keyword in the document title
my $title_weight=10;
# How important it is to find a keyword in the document content
my $content_weight=1;

# Set this to 1 if you want readable but 4*size indexes and an index
# of the contents of each file. Normally set to 0.
my $debug=0;

# You shouldn't have to change anything below this line
#######################################################################

my %index;
my %freq;
my %file_index;

print "Perlfect Indexer\n----------------\n";
&find_valid_files([$base_dir]);
&create_index;
&flatten_index;

open(FILEINDEX, ">$file_index_file") or die "Cannot open $file_index_file";
print FILEINDEX Data::Dumper->Dump([\%file_index],[qw(rfile_index)]);
close(FILEINDEX);
print "File index stored in :$file_index_file\n";

if ($debug) {
  open(INDEX, ">$index_file") or die "Cannot open $index_file";
  print INDEX Dumper(\%index);
  close(INDEX);
  print "Expanded index stored in $index_file\n";
  
  $Data::Dumper::Indent=3;  #pretty printing of data structures
} else {
  $Data::Dumper::Indent=0;  #compact printing of data structures
}

open(FLATINDEX, ">$flat_index_file") or die "Cannot open $flat_index_file";
print FLATINDEX Data::Dumper->Dump([\%freq],[qw(rfreq)]);
close(FLATINDEX);
print "Word(flat) index stored in $flat_index_file\n";

sub find_valid_files {
#takes as an argument a ref to array which contains the dirs to be scanned
  my $dir;
  my $cdir;
  my @dir_contents;
  my @valid_dirs;
  my $full_path;
  local $f_index;

  $f_index=0;

  foreach $dir (@{$_[0]}) {
    if (not &member($dir, \@exclude_dirs)) {
      opendir(BASEDIR, $dir) or die "Cannot open dir $dir";
      @dir_contents=readdir(BASEDIR);
      close(BASEDIR);
      
      print "+ Scanning $dir\n";
      #print "@dir_contents\n";
      foreach $cdir (@dir_contents) {
	$full_path=$dir."/".$cdir;
	if (not -l $full_path and -d $full_path and not $cdir=~/\.{1,2}/) {
	  #print "\tvd: $cdir\n";
	  push @valid_dirs, $full_path;
	} elsif (-f $full_path and $cdir=~/.*\.s{0,1}html$/) {
	  if (not &member($full_path, \@exclude_files)) {
	    $f_index++;
	    $full_path=~s/$base_dir(.*)/$1/;
	    $file_index{$f_index}{file}=$full_path;
	    
	    $index{$f_index}{meta}={};
	    $index{$f_index}{title}={};
	    $index{$f_index}{content}={};
	    print "+\tAdding file $cdir\n";
	  } else {
	    print "-\tSkipping file $cdir\n";
	  }
	}
      }
      &find_valid_files(\@valid_dirs);   
    } else {
      print "- Skipping directory $dir\n";
    }
  }
}

sub create_index {
  my $buffer;
  my $f_index;
  my $file;
  my @valid_files;

  @valid_files=keys %index;
  
  foreach $f_index (sort {$a>=$b} @valid_files) {
    undef $buffer;
    $file=$base_dir.$file_index{$f_index}{file};
    print "Indexing $base_dir$file_index{$f_index}{file} ($f_index)\n";
    open (VFILE, $file) or die "Cannot open $file";
    while(<VFILE>) {
      $buffer.=$_;
    }
    close(VFILE);

    &count_meta($buffer, $f_index);
    &count_title($buffer, $f_index);
    &count_content($buffer, $f_index);
 }
}
  
sub count_meta {
  my $f_index;
  my $buffer;
  my $description;
  my $word;
  my @words;
  my %frequency;

  $buffer=$_[0];
  $f_index=$_[1];

  #print "Counting inside meta tag in file: $file\n";

  if ($buffer=~/<meta\s+name=\"{0,1}description\"{0,1}\s+content=\"(.*?)\">/is) {
    $description=$1;
    $file_index{$f_index}{description}=$description;
    $description=&normalize($description);
    @words=split " ",$description;
    #print "words: @words\n";
    foreach $word (@words) {
      #print "$word\n";
      if ($word) {
	($index{$f_index}{meta}{$word})++;
      }
    }
  } 
}

sub count_title {
  my $f_index;
  my $buffer;
  my $title;
  my $word;
  my @words;
  
  $buffer=$_[0];
  $f_index=$_[1];

  #print "Counting inside title tag in file: $file\n";

  if ($buffer=~/<\s*title\s*>(.*?)<\s*\/title\s*>/is) {
    $title=$1;
    $file_index{$f_index}{title}=$title;
    $title=&normalize($title);
    @words=split " ",$title;
    #print "words: @words\n";
    foreach $word (@words) {
      #print "$word\n";
      if ($word) {
	($index{$f_index}{title}{$word})++;
      }
    }
  } 
}

sub count_content {
  my $f_index;
  my $buffer;
  my $content;
  my $word;
  my @words;
  
  $buffer=$_[0];
  $f_index=$_[1];

  #print "Counting inside content in file: $file\n";

  $buffer=~s/<.*?>//sg;
  $buffer=&normalize($buffer);
  #print $buffer;
  @words=split " ",$buffer;
  #print "words: @words\n";
  foreach $word (@words) {
    #print "$word\n";
    if ($word) {
      ($index{$f_index}{content}{$word})++;
    }
  }
}

sub normalize {
  my $word=$_[0];

  $word=~s/\n/ /g;
  $word=~s/\W/ /g;
  $word=~s/\d+/ /g;
  $word=~s/\b\w\b//g;
  $word=~s/\s+/ /g;
   
  return lc $word;
}

sub flatten_index {
  my $file;
  my $word;

  foreach $file (keys %index) {
    foreach $word (keys %{$index{$file}{meta}}) {
      $freq{$word}{$file}=$meta_weight*$index{$file}{meta}{$word};
    }
    foreach $word (keys %{$index{$file}{title}}) {
      $freq{$word}{$file}=$title_weight*$index{$file}{title}{$word};
    }
    foreach $word (keys %{$index{$file}{content}}) {
      $freq{$word}{$file}=$content_weight*$index{$file}{content}{$word};
    }
  }  
}

sub member {
  my $arg=$_[0];
  my @ary=@{$_[1]};
  my $rv=0;
  my $member;

  foreach $member (@ary) {
    if ($member eq $arg) {
      $rv=1;
    }
  }
  
  return $rv;
}
